<?php

namespace App\Http\Controllers;

use App\Setting;
use App\Traits\SettingTrait;
use Carbon\Carbon;
use Illuminate\Http\Request;

use App\Http\Requests;
use Validator;
use App\Company;
use App\User;
use DB;

class AccountController extends Controller
{
    use SettingTrait;

    public function __construct()
    {
        //
    }

    //Credit a customer
    public function customerCredit(Request $request){
        $transby = [];
        $transby['user_id'] = null;
        $transby['name'] = "admin";
        $transby['user_email'] = null;
        $transby['company_id'] = null;


        $validator = Validator::make($request->all(), [
            'client_number' => 'required',
            'member_id' => 'required',
            'point' => 'required'
        ]);
        if ($validator->fails()) {
            return response()->json(['data' => $validator->messages(), 'status' => '400'], 400);
        }
        $companyid = Company::where('client_number', $request->client_number)->pluck('id');
        if(count($companyid) == 0){
            return response()->json(['data' => 'Company does not exist', 'status' => '400'], 400);
        }
        $currency = $this->currency($companyid[0]);

        if(count($companyid) == 0){
            return response()->json(['data' => 'Company does not exist', 'status' => '400'], 400);
        }
        $userExists = User::where([
            ['id', $request->member_id],
            ['company_id', $companyid[0]]
        ])->count();

        $old_balance = \App\Accounts::where([
            ['user_id', '=', $request->member_id],
            ['company_id', '=', $companyid[0] ],
        ])->select('point')->first();

        try{
            if($userExists > 0){

                DB::beginTransaction();

                //Update the member's account
                \App\Accounts::where([
                    ['user_id', '=', $request->member_id],
                    ['company_id', '=', $companyid[0] ],
                ])->increment('point', $request->point);

                $balance =  \App\Accounts::where([
                    ['user_id', '=', $request->member_id],
                    ['company_id', '=', $companyid[0] ],
                ])->select('point')->first();


                $transaction = new \App\Transaction();
                $transaction->type = $this->transtype['credit'];
                $transaction->type_name = array_keys($this->transtype)[1];
                $transaction->is_internal_action = $this->is_internal_action['external'];
                $transaction->is_internal_action_name = array_keys($this->is_internal_action)[0];
                $transaction->order_id = null;
                $transaction->point_raw = $request->point;
                $transaction->balance_raw = $balance->point;
                $transaction->balance_virtual = $balance->point * (!$currency->is_currency_fixed ? $currency->rate : 1);
                $transaction->company_id = $companyid[0];
                $transaction->member_id = $request->member_id;
                $transaction->point_virtual = $request->point * (!$currency->is_currency_fixed ? $currency->rate : 1);
                $transaction->reason = $request['reason'];
                $transaction->trans_by = json_encode($transby);
                $transaction->save();


                //Check if a total record for the member exists
                $totalExists =  \App\MemberTotal::where(
                    [
                        ['member_id', $request->member_id],
                        ['company_id', $companyid[0]]
                    ]
                )->count();

                //Check if total for the member exists before updating the user's total credit
                if ($totalExists > 0){
                    //Update totals if a record exists
                    \App\MemberTotal::where(
                        [
                            ['member_id', $request->member_id],
                            ['company_id', $companyid[0]]
                        ]
                    )->increment('credit', $request->point);
                }else{
                    //Create a new record if it doesnt exist
                    \App\MemberTotal::firstOrCreate(
                        [
                            'member_id' => $request->member_id,
                            'company_id' => $companyid[0],
                            'credit' => $request->point
                        ]
                    );
                }

                DB::commit();

                //Get the member and and current point
                $customersAccount = \App\User::where('users.id', "=", $request->member_id)
                    ->select(
                        'users.firstname',
                        'users.lastname',
                        'users.email',
                        'users.company_id',
                        'users.id as member_id'
                    )
                    ->first();

                $response = [
                    'firstname' => $customersAccount->firstname,
                    'lastname' => $customersAccount->lastname,
                    'email' => $customersAccount->email,
                    'company_id' => $customersAccount->company_id,
                    'member_id' => $customersAccount->member_id,
                    'credit_raw' => $request->point,
                    'credit_virtual' => $request->point * (!$currency->is_currency_fixed ? $currency->rate : 1),
                    'member_number' => $customersAccount->member_id,
                    'balance_raw' => $transaction->balance_raw,
                    'balance_virtual' => $transaction->balance_virtual,
                    'old_balance_raw' => $old_balance->point,
                    'old_balance_virtual' => $old_balance->point * (!$currency->is_currency_fixed ? $currency->rate : 1),
                    'type_name' => $transaction->type_name,
                ];

                return response()->json(['data' => $response, 'status' => '200'], 200);

            }else{
                return response()->json(['data' => 'Member does not exist', 'status' => '400'], 400);
            }

        }catch (\PDOException  $e){
            DB::rollBack();
            return response()->json(['data' => "Failed to Credit Member",'reason' => $e->getMessage(), 'status' => '400'], 400);
        }
    }

    //Debit a customer
    public function customerDebit(Request $request){
        $transby = [];
        $transby['user_id'] = null;
        $transby['name'] = "admin";
        $transby['user_email'] = null;
        $transby['company_id'] = null;
        $validator = Validator::make($request->all(), [
            'client_number' => 'required',
            'member_id' => 'required',
            'point' => 'required'
        ]);
        if ($validator->fails()) {
            return response()->json(['data' => $validator->messages(), 'status' => '400'], 400);
        }

        $companyid = Company::where('client_number', $request->client_number)->pluck('id');
        if(count($companyid) == 0){
            return response()->json(['data' => 'Company does not exist', 'status' => '400'], 400);
        }
        $currency = $this->currency($companyid[0]);

        $userExists = User::where([
            ['id', $request->member_id],
            ['company_id', $companyid[0]]
        ])->count();

        $old_balance = \App\Accounts::where([
            ['user_id', '=', $request->member_id],
            ['company_id', '=', $companyid[0] ],
        ])->select('point')->first();

        try{
            if($userExists > 0){
                DB::beginTransaction();
                $account =  \App\Accounts::where([
                    ['user_id', '=', $request->member_id],
                    ['company_id', '=', $companyid[0] ],
                ])->decrement('point', $request->point);

                $balance =  \App\Accounts::where([
                    ['user_id', '=', $request->member_id],
                    ['company_id', '=', $companyid[0] ],
                ])->select('point')->first();

                $transaction = new \App\Transaction();
                $transaction->type = $this->transtype['debit'];
                $transaction->type_name = array_keys($this->transtype)[0];
                $transaction->is_internal_action = $this->is_internal_action['external'];
                $transaction->is_internal_action_name = array_keys($this->is_internal_action)[0];
                $transaction->order_id = null;
                $transaction->point_raw = $request->point;
                $transaction->point_virtual = $request->point * (!$currency->is_currency_fixed ? $currency->rate : 1);
                $transaction->balance_raw = $balance->point;
                $transaction->balance_virtual = $balance->point * (!$currency->is_currency_fixed ? $currency->rate : 1);
                $transaction->company_id = $companyid[0];
                $transaction->member_id = $request->member_id;
                $transaction->reason = $request['reason'];
                $transaction->trans_by = json_encode($transby);
                $transaction->save();


                //Check if a total record for the member exists
                $totalExists =  \App\MemberTotal::where(
                    [
                        ['member_id', $request->member_id],
                        ['company_id', $companyid[0]]
                    ]
                )->count();

                //Check if total for the member exists before updating the user's total credit
                if ($totalExists > 0){
                    //Update totals if a record exists
                    \App\MemberTotal::where(
                        [
                            ['member_id', $request->member_id],
                            ['company_id', $companyid[0]]
                        ]
                    )->increment('credit', $request->point);
                }else{
                    //Create a new record if it doesnt exist
                    \App\MemberTotal::firstOrCreate(
                        [
                            'member_id' => $request->member_id,
                            'company_id' => $companyid[0],
                            'credit' => $request->point
                        ]
                    );
                }

                DB::commit();

                if($account){
                    $customersAccount = \App\User::where('users.id', "=", $request->member_id)
                        ->select(
                            'users.firstname',
                            'users.lastname',
                            'users.email',
                            'users.company_id',
                            'users.id as member_id',
                            'users.member_id as member_number'
                        )
                        ->first();

                    $response = [
                        'firstname' => $customersAccount->firstname,
                        'lastname' => $customersAccount->lastname,
                        'email' => $customersAccount->email,
                        'company_id' => $customersAccount->company_id,
                        'member_id' => $customersAccount->member_id,
                        'debit_raw' => $request->point,
                        'debit_virtual' => $request->point * (!$currency->is_currency_fixed ? $currency->rate : 1),
                        'member_number' => $customersAccount->member_id,
                        'balance_raw' => $transaction->balance_raw,
                        'balance_virtual' => $transaction->balance_virtual,
                        'old_balance_raw' => $old_balance->point,
                        'old_balance_virtual' => $old_balance->point * (!$currency->is_currency_fixed ? $currency->rate : 1),
                        'type_name' => $transaction->type_name,
                    ];
                    return response()->json(['data' => $response, 'status' => '200'], 200);
                }else{
                    return response()->json(['data' => 'Failed to update Account', 'status' => '200'], 200);
                }

            }else{
                return response()->json(['data' => 'Member does not exist', 'status' => '400'], 400);
            }

        }catch (\Exception $e){
            DB::rollBack();
            return response()->json(['data' => "Failed to Debit Member",'reason' => $e->getMessage(), 'status' => '400'], 400);
        }
    }

    public function customerAccountTotal(Request $request){
        $validator = Validator::make($request->all(), [
            'client_number' => 'required',
            'member_id' => 'required'
        ]);
        if ($validator->fails()) {
            return response()->json(['data' => $validator->messages(), 'status' => '400'], 400);
        }

        $companyid = Company::where('client_number', $request->client_number)->pluck('id');

        if(count($companyid) == 0){
            return response()->json(['data' => 'Company does not exist', 'status' => '400'], 400);
        }
        $userExists = User::where([
            ['id', $request->member_id],
            ['company_id', $companyid[0]]
        ])->count();

        if($userExists > 0){
            $currentBalance = \App\Accounts::where([
                ['company_id', '=', $companyid[0] ],
                ['user_id', '=', $request->member_id ]
            ])->select('point as current_balance')->get();
            return response()->json(['data' => $currentBalance, 'status' => '200'], 200);
        }else{
            return response()->json(['data' => 'User does not exist', 'status' => '400'], 400);
        }

    }

    public function getMemberPointTotalCredit(Request $request){
        $validator = Validator::make($request->all(), [
            'client_number' => 'required',
            'member_id' => 'required'
        ]);
        if ($validator->fails()) {
            return response()->json(['data' => $validator->messages(), 'status' => '400'], 400);
        }

        $companyid = Company::where('client_number', $request->client_number)->pluck('id');

        if(count($companyid) == 0){
            return response()->json(['data' => 'Company does not exist', 'status' => '400'], 400);
        }
        $userExists = User::where([
            ['id', $request->member_id],
            ['company_id', $companyid[0]]
        ])->count();

        if($userExists > 0){
            $totalCredit = \App\MemberTotal::where([
                ['company_id', '=', $companyid[0] ],
                ['member_id', '=', $request->member_id ]
            ])->select('credit')->get();
            return response()->json(['data' => $totalCredit, 'status' => '200'], 200);
        }else{
            return response()->json(['data' => 'Company does not exist', 'status' => '400'], 400);
        }
    }

    public function getMemberPointTotalDebit(Request $request){
        $validator = Validator::make($request->all(), [
            'client_number' => 'required',
            'member_id' => 'required'
        ]);
        if ($validator->fails()) {
            return response()->json(['data' => $validator->messages(), 'status'=>'400'], 400);
        }

        $companyid = Company::where('client_number', $request->client_number)->pluck('id');

        if(count($companyid) == 0){
            return response()->json(['data' => 'Company does not exist', 'status' => '400'], 400);
        }
        $userExists = User::where([
            ['id', $request->member_id],
            ['company_id', $companyid[0]]
        ])->count();

        if($userExists > 0){
            $totalCredit = \App\MemberTotal::where([
                ['company_id', '=', $companyid[0] ],
                ['member_id', '=', $request->member_id ]
            ])->select('debit')->get();
            return response()->json(['data' => $totalCredit, 'status' => '200'], 200);
        }else{
            return response()->json(['data' => 'User does not exist', 'status' => '400'], 400);
        }
    }

    public function getCustomersPlusAccount(Request $request){
        $customersAccount = \App\User::join('accounts', function ($join){
            $join->on('users.id', '=', 'accounts.user_id');
            $join->on('accounts.company_id', '=', 'users.company_id');
        })
            ->select(
                'users.firstname',
                'users.lastname',
                'users.email',
                'users.company_id',
                'users.phone',
                'users.address',
                'accounts.point as account'
            )
            ->get();
        return response()->json(['data' => $customersAccount, 'status' => '200'], 200);
    }

}