<?php

namespace App\Http\Controllers;

use App\Setting;
use Carbon\Carbon;
use Illuminate\Http\Request;

use App\Http\Requests;
use Validator;
use App\Company;
use App\User;
use DB;
use App\Traits\SettingTrait;
use App\Http\Validators\ValidateInput;
use App\Repository\UserRepository;
use App\Http\Controllers\MobileNoController;
use App\Http\Controllers\CustomerController;
class CompanyController extends Controller
{

    use SettingTrait;

    public function __construct()
    {
        //
    }


    public function CreateCompany(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'name' => 'required',
            'token' => 'required|unique:company,token',
            'client_number' => 'required'
        ]);
        if ($validator->fails()) {
            return response()->json(['data' => $validator->messages(), 'status' => '400'], 400);
        }

        //Check if Company exists
        $companyExists = Company::where([
                ['token', '=', $request->token],
                ['client_number', '=', $request->client_number]
            ]
        )->count();


        if(!$companyExists) {
            DB::beginTransaction();
            try {
                $company = new Company();
                $company->name = $request->name;
                $company->token = $request->token;
                $company->client_number = $request->client_number;
                $company->save();

                $setting = new Setting();
                $setting->company_id = $company->id;
                $setting->name = $company->name;
                $setting->rate = 1;
                $setting->currency = "points";
                $setting->is_currency_fixed = 1;
                $setting->has_api_access = 0;
                $setting->save();

                if(!$company || !$setting){
                    DB::rollBack();
                }

                DB::commit();
                return response()->json(['data' => ['company' =>$company, 'setting' => $setting], 'status' => '200'], 200);
            } catch (\Exception $e) {
                DB::rollBack();
                return response()->json(['data' => "An Error Occurred" . $e->getMessage(), 'status' => '400'], 400);
            }
        }else{
            return response()->json(['data' => "Company already registered", 'status' => '400'], 400);
        }
    }

    public function DeleteCompany(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'client_number' => 'required'
        ]);

        if ($validator->fails()) {
            return response()->json(['data' => $validator->messages(), 'status' => '400'], 400);
        }

        //Check if Company exists
        $company = Company::where([
                ['client_number', '=', $request->client_number]
            ]
        )->first();


        if(count($company)) {
           Company::where([
                    ['client_number', '=', $request->client_number]
                ]
            )->delete();
                return response()->json(['data' => 'Company deleted', 'status' => '200'], 200);
        }else{
            return response()->json(['data' => "Company does not exist", 'status' => '400'], 400);
        }
    }


    public function getCompanyCustomer(Request $request){

        $validator = Validator::make($request->all(), [
            'client_number' => 'required',
            'member_number' => 'required'
        ]);
        if ($validator->fails()) {
            return response()->json(['data' => $validator->messages(), 'status' => '400'], 400);
        }
        $company = Company::where('client_number', $request->client_number)->select('id')->first();
        if(count($company) == 0){
            return response()->json(['data' => 'Company does not exist', 'status' => '400'], 400);
        }
        $currency = $this->currency($company->id);

        $customers = DB::table('users')
            ->join('accounts', 'users.id', '=', 'accounts.user_id')
            ->select(
                'users.id as member_id',
                'users.firstname',
                'users.lastname',
                'users.member_id as member_number',
                'users.phone',
                'users.status',
                'users.address',
                'users.company_id',
                'users.email',
                'users.created_at',
                'accounts.point as point_raw'
            )
            ->addSelect(DB::raw(" (accounts.point * $currency->rate) as point_virtual") )
            ->where('users.company_id', $company->id)
            ->where('users.member_id', $request->member_number)
            ->first();
            //dd($customers);
        if ($customers == null){

            return response()->json(['data' => 'Member Not found', 'status' => '400'], 400);
        } else {
            
            
        
        return response()->json(['data' => $customers, 'status' => '200'], 200);
        }

    }

    public function getCompanyCustomers(Request $request){

        $validator = Validator::make($request->all(), [
            'client_number' => 'required'
        ]);
        if ($validator->fails()) {
            return response()->json(['data' => $validator->messages(), 'status' => '400'], 400);
        }

        $company = Company::where('client_number', $request->client_number)->select('id');

        if(!$company->exists()){
            return response()->json(['data' => 'Company does not exist', 'status' => '400'], 400);
        }

        $company = $company->first();

        $currency = $this->currency($company->id);

        $customers = \App\User::where('users.company_id', $company->id);
        $customers = $customers->where('users.status', 1);



        if ($request->has('member_id')) {

            if ($request->has('from_date') and $request->has('to_date')) {
                $fromDate = $request->from_date." 00:00:00";
                $toDate = $request->to_date." 23:59:59";
                $customers = $customers->whereBetween('users.created_at', [$fromDate, $toDate]);
            }

            $customers = $customers->where('users.id', $request['member_id'])
                ->join('accounts', 'users.id', '=', 'accounts.user_id')
                ->select(
                    'users.id as member_id',
                    'users.firstname',
                    'users.lastname',
                    'users.member_id as member_number',
                    'users.phone',
                    'users.status',
                    'users.address',
                    'users.company_id',
                    'users.email',
                    'users.created_at',
                    'accounts.point as point_raw'
                )
                ->addSelect(DB::raw(" (accounts.point * $currency->rate) as point_virtual") )
                ->first();
            return response()->json(['data' => $customers, 'status' => '200'], 200);
        }

        if ($request->has('member_ids')) {
            if(!is_array($request->input('member_ids'))){
                return response()->json(
                    [
                        'data' => [],
                        'message' => 'member_ids has to be an array',
                        'status' => '400',
                    ], 400);
            }

            if ($request->has('from_date') and $request->has('to_date')) {
                $fromDate = $request->from_date." 00:00:00";
                $toDate = $request->to_date." 23:59:59";
                $customers = $customers->whereBetween('users.created_at', [$fromDate, $toDate]);
            }

            $customers = $customers->whereIn('users.id', $request['member_ids']);

            $customers = $customers->leftJoin('accounts', 'users.id', '=', 'accounts.user_id')
                ->select(
                    'users.id as member_id',
                    'users.firstname',
                    'users.lastname',
                    'users.member_id as member_number',
                    'users.phone',
                    'users.status',
                    'users.address',
                    'users.company_id',
                    'users.email',
                    'users.created_at',
                    'accounts.point as point_raw'
                )
                ->addSelect(DB::raw(" (accounts.point * $currency->rate) as point_virtual") )
                ->get();

            return response()->json(['data' => $customers, 'status' => '200'], 200);
        }

        $customers = $customers->get();
        return response()->json(['data' => $customers, 'status' => '200'], 200);

    }

    public function getCompanyData(Request $request){
        $validator = Validator::make($request->all(), [
            'client_number' => 'required',
        ]);

        if ($validator->fails()) {
            return response()->json(['data' => $validator->messages(), 'status' => '400'], 400);
        }

        $company = Company::where('client_number', $request->client_number)->select(  'name',
            'token',
            'client_number',
            'has_api_access',
            'created_at',
            'id'
        )->first();


        if(count($company) == 0){
            return response()->json(['data' => 'Company does not exist', 'status' => '400'], 400);
        }

        return response()->json(['data' => $company, 'status' => '200'], 200);
    }

    public function getCompanyTransaction(Request $request){

        if (!$request->has('client_number')) {
            return response()->json(['data' => 'client_number is required', 'status' => '400'], 400);
        }

        $company = Company::where('client_number', $request->client_number)->select('id');

        if(!$company->exists()){
            return response()->json(['data' => 'Company record not found', 'status' => '400'], 400);
        }

        $company = $company->first();
        $transaction = new \App\Transaction();

        if($request->has('from_date') && $request->has('to_date') ) {
            $transaction = $transaction->whereBetween(
                'transactions.created_at', [$request->from_date, $request->to_date]);
        }


       if ($request->has('member_ids')){
            $transaction = $transaction->where([
                    ['transactions.company_id', '=', $company->id]
                ])
                ->whereIn('transactions.member_id', $request['member_ids']);
        }
        elseif ($request->has('member_id')) {
            $transaction = $transaction->where([
                    ['transactions.company_id', '=', $company->id]
                ])
                ->where('transactions.member_id', $request['member_id']);

        }
        else {
           $transaction = $transaction->where('transactions.company_id', $company->id);
       }
            $transaction =  $transaction
                ->leftJoin('orders', 'transactions.order_id', '=', 'orders.id')
                ->leftJoin('users', 'transactions.member_id', '=', 'users.id')
                ->select('order_no', 'balance_virtual',
                    'transactions.balance_raw',
                    'transactions.point_virtual',
                    'transactions.point_raw',
                    'transactions.trans_by',
                    'transactions.member_id',
                    'transactions.company_id',
                    'transactions.order_id',
                    'orders.order_no',
                    'transactions.is_internal_action_name',
                    'transactions.is_internal_action',
                    'transactions.type_name',
                    'transactions.type',
                    'transactions.created_at',
                    'transactions.updated_at',
                    'transactions.id as transaction_id',
                    'transactions.reason',
                    'orders.is_shipping',
                    'orders.shipping_cost',
                    'users.member_id as member_no',
                    'users.firstname as firstname',
                    'users.lastname as lastname'
                )
                ->get();

            return response()->json(['data' => $transaction, 'status' => '200'], 200);
        }


    public function appCurrency(Request $request){
        $validator = Validator::make($request->all(), [
            'client_number' => 'required',
        ]);
        if ($validator->fails()) {
            return response()->json(['data' => $validator->messages(), 'status' => '400'], 400);
        }
        try {
            $company = Company::where('client_number', $request->client_number)->select('id')->first();
            if(count($company) == 0){
                return response()->json(['data' => 'Company does not exist', 'status' => '400'], 400);
            }

            $currency = DB::table('setting')
                ->join('company', 'setting.company_id', '=', 'company.id')
                ->where('company_id', $company->id)
                ->select('company.name as company_name', 'setting.rate', 'setting.currency', 'setting.is_currency_fixed')->first();

            return response()->json(['data' => $currency, 'status' => '200'], 200);

        }catch (\Exception $e){
            return response()->json(['data' => 'Error getting Application Currency data'.$e->getMessage(), 'status' => '400'], 400);
        }
    }

    public function updateCurrency(Request $request){
        $validator = Validator::make($request->all(), [
            'rate' => 'required',
            'currency' => 'required',
            'is_currency_fixed' => 'required',
            'client_number' => 'required',
        ]);
        if ($validator->fails()) {
            return response()->json(['data' => $validator->messages(), 'status' => '400'], 400);
        }

        $company = Company::where('client_number', $request->client_number);

        if(!$company->exists()){
            return response()->json(['data' => 'Company does not exist', 'status' => '400'], 400);
        }

        $company = $company->select('id')->first();

        try {
            DB::table('setting')
                ->where('company_id', $company->id)
                ->update([
                    'rate' => $request->rate,
                    'currency' => $request->currency,
                    'is_currency_fixed' => $request->is_currency_fixed
                ]);

            $newsetting = DB::table('setting')
                ->join('company', 'setting.company_id', '=', 'company.id')
                ->where('company_id', $company->id)
                ->select('company.name as company_name', 'setting.rate', 'setting.currency', 'setting.is_currency_fixed')->first();
            return response()->json(['message' => 'Success Settings updated', 'data' => $newsetting, 'status' => '200'], 200);
        }catch (\Exception $e){
            return response()->json(['data' => 'An error occurred' . $e->getMessage(), 'status' => '400'], 400);
        }
    }

    public function convertCurrency(Request $request){
        $validator = Validator::make($request->all(), [
            'point' => 'required',
            'client_number' => 'required',
        ]);
        if ($validator->fails()) {
            return response()->json(['data' => $validator->messages(), 'status' => '400'], 400);
        }
        try {
            $company = Company::where('client_number', $request->client_number)->select('id')->first();
            if(count($company) == 0){
                return response()->json(['data' => 'Company does not exist', 'status' => '400'], 400);
            }
            $currency = DB::table('setting')
                ->where('company_id', $company->id)
                ->select(
                    'rate',
                    'currency'
                )
                ->first();
            $data = [
                'point_virtual' => ($request->point * $currency->rate),
                'currency' => $currency->currency
            ];
            return response()->json(['data' => $data, 'status' => '200'], 200);
        }catch (\Exception $e){
            return response()->json(['data' => 'An error occurred' . $e->getMessage(), 'status' => '400'], 400);
        }
    }

    public function setCompanyApiStatus(Request $request){
        $validator = Validator::make($request->all(), [
            'status' => 'required',
            'client_number' => 'required',
        ]);
        if ($validator->fails()) {
            return response()->json(['data' => $validator->messages(), 'status' => '400'], 400);
        }

        $company = DB::table('company')
            ->where('client_number', $request['client_number'])
            ->leftJoin('setting','company.id', '=', 'setting.company_id')
            ->select('setting.has_api_access', 'company.name as company_name', 'company.id as company_id')->first();
//        return response()->json($company->has_api_access );

        $response_msg = $request['status'] == 1 ? "Access granted to company" : "Access restriction enabled";

        if(!$company) {
            return response()->json(['message' => 'Company does not exist', 'status' => '400'], 400);
        }else if($request['status'] == 1 && $company->has_api_access == 1) {
            return response()->json(['message' => 'No changes effected. '.$company->company_name.' has already been granted API Access.', 'status' => '200'], 200);
        }else if($request['status'] == 0 && $company->has_api_access == 0) {
            return response()->json(['message' => 'No changes effected. '.$company->company_name.' already has API Access Restriction.', 'status' => '200'], 200);
        }else
        {
            try {
                DB::table('setting')
                    ->where('company_id', $company->company_id)
                    ->update(
                        ['has_api_access' => $request['status']]
                    );
                return response()->json(['message' => $response_msg, 'status' => '200'], 200);
            }catch (\Exception $e){
                return response()->json(['message' => 'Failed to update company API access status ' .$e->getMessage(), 'status' => '400'], 400);
            }
        }

    }

    public function getCompanyApiStatus(Request $request){
        $validator = Validator::make($request->all(), [
            'company_id' => 'required',
        ]);
        if ($validator->fails()) {
            return response()->json(['data' => $validator->messages(), 'status' => '400'], 400);
        }

        $companyExists = DB::table('setting')
            ->where('company_id', $request['company_id'])
            ->exists();

        if(!$companyExists) {
            return response()->json(['message' => 'Company does not exist', 'status' => '400'], 400);
        }else{
            try {
                $hasAccess = DB::table('setting')
                    ->where('company_id', $request['company_id'])
                    ->select('has_api_access')->first();
                return response()->json(['message' => $hasAccess->has_api_access, 'status' => '200'], 200);
            }catch (\Exception $e){
                return response()->json(['message' => 'Failed to get company access status ' .$e->getMessage(), 'status' => '400'], 400);
            }
        }

    }

    public function setCompanyPinStatus(Request $request){
        $validator = Validator::make($request->all(), [
            'status' => 'required',
            'client_number' => 'required',
        ]);
        if ($validator->fails()) {
            return response()->json(['data' => $validator->messages(), 'status' => '400'], 400);
        }

        $company = DB::table('company')
            ->where('client_number', $request['client_number'])
            ->leftJoin('setting','company.id', '=', 'setting.company_id')
            ->select('setting.has_pin_validation', 'company.name as company_name', 'company.id as company_id')->first();

        $response_msg = $request['status'] == 1 ? "PIN Access Activated" : "PIN Access Deactivated";

        if(!$company) {
            return response()->json(['message' => 'Company does not exist', 'status' => '400'], 400);
        }else if($request['status'] == 1 && $company->has_pin_validation == 1) {
            return response()->json(['message' => 'No changes effected. '.$company->company_name.' has already been granted PIN Access.', 'status' => '200'], 200);
        }else if($request['status'] == 0 && $company->has_pin_validation == 0) {
            return response()->json(['message' => 'No changes effected. '.$company->company_name.' already has PIN Access Deactivated.', 'status' => '200'], 200);
        }else
        {
            try {
                DB::table('setting')
                    ->where('company_id', $company->company_id)
                    ->update(
                        ['has_pin_validation' => $request['status']]
                    );
                return response()->json(['message' => $response_msg, 'status' => '200'], 200);
            }catch (\Exception $e){
                return response()->json(['message' => 'Failed to update company PIN Access status ' .$e->getMessage(), 'status' => '400'], 400);
            }
        }
    }


    public function getCompanyCustomersWithPoints(Request $request){

        $validator = Validator::make($request->all(), [
            'client_number' => 'required'
        ]);
        if ($validator->fails()) {
            return response()->json(['data' => $validator->messages(), 'status' => '400'], 400);
        }

        $company = Company::where('client_number', $request->client_number)->select('id');

        if(!$company->exists()){
            return response()->json(['data' => 'Company does not exist', 'status' => '400'], 400);
        }

        $company = $company->first();

        $currency = $this->currency($company->id);

        $customers = \App\User::where('users.company_id', $company->id);






            $customers = $customers->where('accounts.point','>', 0)
                ->join('accounts', 'users.id', '=', 'accounts.user_id')
                ->select(
                    'users.id as member_id',
                    'users.firstname',
                    'users.lastname',
                    'users.member_id as member_number',
                    'users.status',
                    'users.company_id',
                    'users.email',
                    'accounts.point as point_raw'
                )
                ->addSelect(DB::raw(" (accounts.point * $currency->rate) as point_virtual") )
                ->get();
            return response()->json(['data' => $customers, 'status' => '200'], 200);




    }
    
    
    public function forfiet_all_user_points (Request $request){

        $validator = Validator::make($request->all(), [
            'client_number' => 'required'
        ]);
        if ($validator->fails()) {
            return response()->json(['data' => $validator->messages(), 'status' => '400'], 400);
        }
        //$company = Company::where('client_number', $client_no)->select('id');
        $company = Company::where('client_number', $request->client_number)->select('id');
        if(!$company->exists()){
            return response()->json(['data' => 'Company does not exist', 'status' => '400'], 400);
        }

        $company = $company->first();

        $currency = $this->currency($company->id);

        $customers = \App\User::where('users.company_id', $company->id);






        $customers = $customers->where('accounts.point','>', 0)
            ->join('accounts', 'users.id', '=', 'accounts.user_id')
            ->select(
                'users.id as member_id',
                'users.firstname',
                'users.lastname',
                'users.member_id as member_number',
                'users.status',
                'users.company_id',
                'users.email',
                'accounts.point as point_raw'
            )
            ->addSelect(DB::raw(" (accounts.point * $currency->rate) as point_virtual") )
            ->get();

        if (count($customers) > 0){
            $use_customer = new CustomerController();
            foreach ($customers as $members){
                $debit_customer =$use_customer->DebitCustomerPoints($request->client_number, $members['member_id'],$members['point_raw'], "Points Forfeiture");
               //$debit_customer = $use_customer->DebitCustomerPoints($client_no, $members['member_id'],$members ['point_raw'], "Points Forfeiture");


            }

            return response()->json(['data' => $customers, 'status' => '200'], 200);
        } else {

            return response()->json(['message' => 'No member found', 'status' => '400'], 400);
        }


    }
    
    public function getCompanyCustomersDisabled(Request $request)
    {

        $validator = Validator::make($request->all(), [
            'client_number' => 'required'
        ]);
        if ($validator->fails()) {
            return response()->json(['data' => $validator->messages(), 'status' => '400'], 400);
        }

        $company = Company::where('client_number', $request->client_number)->select('id');

        if (!$company->exists()) {
            return response()->json(['data' => 'Company does not exist', 'status' => '400'], 400);
        }

        $company = $company->first();

        $currency = $this->currency($company->id);

        $get_customers = DB::table('users')
            ->where('users.company_id', $company->id)
            ->select(
                'users.id as member_id',
                'users.firstname',
                'users.lastname',
                'users.member_id as member_number',
                'users.status',
                'users.company_id',
                'users.email',
                'users.phone',
                'accounts.point as point_raw'
            )

            ->leftjoin('accounts', 'users.id', '=', 'accounts.user_id')
            ->addSelect(DB::raw(" (accounts.point * $currency->rate) as point_virtual") )
            ->where('users.status', 0)
            ->get();

        return response()->json(['data' => $get_customers, 'status' => '200'], 200);
    }



    public function getCompanyCustomersEnabledNoLogin(Request $request)
    {

        $validator = Validator::make($request->all(), [
            'client_number' => 'required'
        ]);
        if ($validator->fails()) {
            return response()->json(['data' => $validator->messages(), 'status' => '400'], 400);
        }

        $get_data =  $this->get_customer_no_login($request->client_number);
        return response()->json(['data' => $get_data, 'status' => '200'], 200);
    }

    private function get_customer_no_login($client_number){

        $company = Company::where('client_number', $client_number)->select('id');

        if (!$company->exists()) {
            return response()->json(['data' => 'Company does not exist', 'status' => '400'], 400);
        }

        $company = $company->first();

        $currency = $this->currency($company->id);

        $get_customers = DB::table('users')
            ->where('users.company_id', $company->id)
            ->select(
                'users.id as member_id',
                'users.firstname',
                'users.lastname',
                'users.member_id as member_number',
                'users.status',
                'users.company_id',
                'users.email',
                'users.phone',
                'accounts.point as point_raw'
            )

            ->leftjoin('accounts', 'users.id', '=', 'accounts.user_id')
            ->addSelect(DB::raw(" (accounts.point * $currency->rate) as point_virtual") )
            ->where('users.status', 1)
            ->where('users.first_login', 0)
            // ->where('users.created_at','>', '2020-07-28')
            // ->where('users.created_at','<', '2020-08-01')
            ->get();

        return $get_customers;
    }

    public function getCompanyCustomersNologinSms(Request $request)
    {

        $validator = Validator::make($request->all(), [
            'client_number' => 'required'
        ]);
        if ($validator->fails()) {
            return response()->json(['data' => $validator->messages(), 'status' => '400'], 400);
        }

        $get_data =  $this->get_customer_no_login($request->client_number);
        //return response()->json(['data' => $get_data, 'status' => '200'], 200);
        if($get_data){
            
            $use_mobile = new MobileNoController();
        
            $use_customer = new CustomerController();
        
            $respond_array = array();
            $respond_array['success'] = array();
            $respond_array['failed'] = array();
           
            foreach($get_data as $datum){
                //dd($datum);
                //return response()->json(['data' => $datum, 'status' => '200'], 200);
                $check_number = $use_mobile->check_mobile_no($datum->phone);

                if(is_array($check_number)){

                    $datum->reason = "Invalid Mobile No";

                    $respond_array['failed'][] = $datum;
                } else {

                    $datum->phone = $check_number;
                    $change_password = $use_customer->MemberChangePassword($datum->member_id);

                    if(is_array($change_password)){
                        $datum->reason = "Error Changing Password";
                        $respond_array['failed'][] = $datum;

                    } else {

                        $datum->password = $change_password;
                        $respond_array['success'][] = $datum;
                    }
                }



            }

            return response()->json(['data' => $respond_array, 'status' => '200'], 200);

        } else {

            return response()->json(['data' => 'No data found', 'status' => '400'], 400);
        }
    }
}