<?php
namespace App\Http\Controllers;
use App\Interfaces\ISections;
use App\Repository\TransactionRepository;
use App\Traits\Email;
use League\Fractal;
use Tymon\JWTAuth\JWTAuth;
use Illuminate\Http\Request;
use App\Http\Requests;

//Import Traits
use App\Traits\User;
use App\Traits\Common;
use App\Traits\EventsTrait;
use App\Traits\SettingTrait;


class EventsController extends ApiController implements ISections
{
    use User, SettingTrait, Common, EventsTrait, Email;

    public $transactionRepository;

    public function __construct(JWTAuth $jwt)
    {
        $this->middleware('auth:api');
        $this->jwt = $jwt;
        $this->transactionRepository = new TransactionRepository();
    }

    public function content(Request $request)
    {

        $currency = $this->currency($this->getUser()->company->id);
        $token = $this->getUser()->company->token;
        $curlService = new \Ixudra\Curl\CurlService();
        $data = $curlService->to(env('BASE_URL').'view_events')
            ->withHeader('token: '.$token)
            ->asJsonResponse()
            ->get();
      
        if(isset($data)){
            if($data){
                if($data->status == 1){
                    $resource = new Fractal\Resource\Collection($data->events, function( $item) use($currency)  {

                        if(isset($item->tickets)){
                            if($currency->is_currency_fixed == '0'){
                                $tickets = [];
                                foreach ($item->tickets as $ticket) {
                                    $tickets[] = (object)[
                                        "price" => $currency->rate * $ticket->price,
                                        "currency_name" => $currency->currency,
                                        "title" => $ticket->title,
                                        "venue" => $ticket->venue,
                                        "signature" => $ticket->signature,
                                    ];
                                }
                            }else{
                                foreach ($item->tickets as $ticket) {
                                    $tickets[] = (object)[
                                        "price" => $ticket->price,
                                        "currency_name" => "naira",
                                        "title" => $ticket->title,
                                        "venue" => $ticket->venue,
                                        "signature" => $ticket->signature,
                                    ];
                                }
                            }

                        }

                        return [
                            'event_title' => $item->title,
                            'event_description' => $item->description,
                            'artwork' => $item->artwork,
                            'banner' => $item->banner,
                            'organizer' => $item->organizer,
                            'organizer_logo' => $item->organizer_logo,
                            'date' => $item->date,
                            'tickets' => $tickets
                        ];
                    });
                    $array = $this->apiResponse()->createData($resource)->toArray();
                    return response()->json(
                        [
                            'message' => $data->message,
                            'data' => $array['data'],
                            'status' => '200'],
                        200);
                }else if($data->status == 0){
                    return response()->json(
                        [
                            'message' => $data->message,
                            'data' => [],
                            'status' => '400'],
                        400);
                }
            }
            return response()->json(
                [
                    'message' => 'Failed to retrive data. Try again.',
                    'data' => [],
                    'status' => '400'],
                400);
        }else{
            return response()->json(
                [
                    'message' => 'Failed to retrive data. Try again.',
                    'data' => [],
                    'status' => '400'],
                400);
        }

    }

    public function redeemEvent(Request $request){


        if (!$request->has('phone_no')) {
            return response()->json([
                'message' => 'Validation error',
                'data' => 'phone_no param is required',
                'status' => '400'], 400);
        }
        if (!$request->has('email')) {
            return response()->json([
                'message' => 'Validation error',
                'data' => 'email param is required',
                'status' => '400'], 400);
        }
        if (!$request->has('signature')) {
            return response()->json([
                'message' => 'Validation error',
                'data' => 'signature param is required',
                'status' => '400'], 400);
        }

        $validator = \Validator::make($request->all(), [
            'phone_no' => 'required|min:10|max:11',
            'email' => 'required|min:5|max:255|email',
            'price' => 'required',
            'qty' => 'required',
            'event_name' => 'required',
        ]);

        if ($validator->fails()) {
            return response()->json([
                'message' => $validator->messages()->first(),
                'status' => '400'], 400);
        }

        $currency = $this->currency($this->getUser()->company->id);

        $price = ($request->price / ($currency->is_currency_fixed == 0 ? $currency->rate : 1) );

        //Check if the user has enough account balance before proceeding with the transaction
        $canpurchase = $this->canPurchase(
            $this->getUser()->id,
            $this->getUser()->company_id,
            (int)($price * $request->qty));

        if(!$canpurchase){
            return response()->json([
                'message' => 'Apologies! Your current balance is insufficient for this transaction.',
                'status' => '400'],
                200);
        }

        if(!$this->deductFromAccount(
            $this->getUser()->id,
            $this->getUser()->company_id,
            ($price * $request->qty))
        ) {
            return response()->json([
                'message' => 'An error occurred. Please Try again.',
                'status' => '400'],
                200);
        }

        //Add to Cart
        $cartItemId = $this->addToCart(
            $this->getUser()->id,
            $this->getUser()->company_id,
            $price,
            $request->event_name,
            $request->signature,
            1
        );

        if(!$cartItemId){
            return response()->json([
                'message' => 'Failed to add item to Cart. Try again',
                'status' => '400'],
                200);
        }

        //Add properties to the array
        $payload['quantity'] = $request->qty;
        $payload['member_no'] = $this->getUser()->member_id;
        $payload['price'] = $price;
        $payload['signature'] = $request->signature;
        $payload['email'] = $request->email;
        $payload['phone_no'] = $request->phone_no;

        $response = $this->checkoutEvent(
            $this->getUser()->company->token,
            $payload
        );

        if($response){
            if($response->status == 0){
                if($this->refundFailedTransToAccount(
                    $this->getUser()->id,
                    $this->getUser()->company_id,
                    ( $price * $request->qty)
                )){
                    $this->deleteFromCart($cartItemId);
                    return response()->json([
                        'message' => $response->message,
                        'status' => '400'],
                        200);
                }else{
                    return response()->json([
                        'message' => 'An error occurred. Refund failed. Contact Support.',
                        'status' => '200'],
                        200);
                }
                return response()->json([
                    'message' => $response->message,
                    'status' => '200'],
                    200);
            }else{

                try {

                    //Add to Order
                    $success = [ ['ref_no' => $cartItemId, "voucher_code" => $response->voucher_code ]];
                    $orderSaveStatus = $this->addOrder(
                        $this->getUser()->id,
                        $this->getUser()->company_id,
                        $cartItemId,
                        $success,
                        $price);
                    //Save Transaction
                    $this->transactionRepository->SaveTransaction(
                        $this->getUser()->id,
                        $this->getUser()->company_id,
                        $this->getUser()->firstname,
                        $this->getUser()->lastname,
                        $this->getUser()->email,
                        $request,
                        $orderSaveStatus,
                        $price,
                        null
                    );

                    if(!$orderSaveStatus){
                        return response()->json(['message' => 'Failed to save Order', 'status' => '400'], 400);
                    }else{
                        $virtual_price = null;
                        if ($currency->is_currency_fixed == '1'){
                            $virtual_price =" '&#8358 '.(number_format($price * $request->qty) ";
                        }
                        else
                        {
                            $virtual_price = number_format($currency->rate * ($price * $request->qty) ) . ' ' . $currency->currency;
                        }

                        $product_details = (object)[
                            'name' =>  $request->event_name,
                            'quantity' =>  '1',
                            'virtual_price' =>  $virtual_price,
                            'delivery_type_name' => 'E-Channel',
                            'voucher' => $response->voucher_code,
                        ];

                        $receipt = $this->buildExpDetails($product_details, $response->order_no);

                        $email_payload = [
                            'first_name' => $this->getUser()->firstname,
                            'current_balance' => $this->getUser()->accountBalanceText(),
                            'email' => $this->getUser()->email,
                            'details' => $receipt
                        ];

                        $token = $this->getUser()->company->token;

                        $this->sendStatusChangeEmail(1, $token, $email_payload);
                        return response()->json([
                            'message' => 'success',
                            'data' => $orderSaveStatus,
                            'status' => '200',
                            'account' => $this->getUser()->accountBalanceText()], 200);
                    }
                }catch (\Exception $e){
                    return response()->json(['message' => 'Failed to record Order ' . $e->getMessage(), 'status' => '400'], 400);
                }
                return response()->json(['message' => $response->message, 'status' => '400'], 400);

            }
        }
        return response()->json([
            'message' => 'No response from the network',
            'status' => '400'],
            200);
    }

}