<?php

use Laravel\Lumen\Testing\DatabaseTransactions;


class CustomerDataTest extends TestCase
{
    public function testGetCompanyCustomerWithoutParameters()
    {
        $this->post('/company/customer/show')
            ->seeStatusCode(400);
    }

    public function testGetCompanyCustomerWithParameters()
    {
        $this->post('/company/customer/show', ['client_number' => 6, 'member_number' => 2111])
            ->seeStatusCode(200)
            ->seeJsonStructure(['data' =>
                [

                    "member_id",
                    "firstname",
                    "lastname",
                    "member_number",
                    "phone",
                    "status",
                    "address",
                    "company_id",
                    "email",
                    "created_at",
                    "point_raw",
                    "point_virtual"
                ]

            ]);
    }

    public function testCustomerTransactions()
    {
        $this->post('/customer/transaction/list', ['client_number' => 6, 'member_id' => 1])
            ->seeStatusCode(200)
            ->seeJsonStructure(
                ['data' =>
                    [
                        '*' => [
                            "trans_id",
                            "trans_type",
                            "type_name",
                            "is_internal_action",
                            "is_internal_action_name",
                            "order_id",
                            "trans_by",
                            "point_raw",
                            "point_virtual",
                            "balance_raw",
                            "balance_virtual",
                            "email",
                            "firstname",
                            "lastname"

                        ]
                    ]
                ]
            );
    }

    public function testPostCustomerStatus()
    {
        $this->post('/customer/status', ['client_number' => 6, 'member_id' => 1, 'status' => 1])
            ->seeStatusCode(200);
    }

    public function testGetCustomerOrder()
    {
        $this->post('/customer/order/list', ['client_number' => 6, 'member_id' => 1, 'status' => 1])
            ->seeStatusCode(200)
            ->seeJsonStructure(
                ['data' =>
                    [
                        '*' => [
                            "id",
                            "user_id",
                            "company_id",
                            "address_id",
                            "sub_total_cost",
                            "is_shipping",
                            "shipping_cost",
                            "success",
                            "fail",
                            "order_no",
                            "created_at",
                            "updated_at"
                        ]
                    ]
                ]
            );
    }

    public function testGetOrderItems()
    {
        $this->post('/customer/order', ['order_id' => 6, 'member_id' => 1, 'status' => 1])
            ->seeStatusCode(200);
    }

    public function testCustomerCredit()
    {
        $this->post('/customer/account/credit', ['client_number' => 6, 'member_id' => 1, 'point' => 100])
            ->seeStatusCode(200)
            ->seeJsonStructure(
                ['data' =>
                    ["firstname",
                        "lastname",
                        "email",
                        "company_id",
                        "member_id",
                        "credit_raw",
                        "credit_virtual",
                        "member_number",
                        "balance_raw",
                        "old_balance_raw",
                        "balance_virtual",
                        "old_balance_virtual",
                        "type_name"
                    ]
                ]
            );
    }

    public function testCustomerDebit()
    {
        $this->post('/customer/account/debit', ['client_number' => 6, 'member_id' => 1, 'point' => 100])
            ->seeStatusCode(200)
            ->seeJsonStructure(
                ['data' =>
                    ["firstname",
                        "lastname",
                        "email",
                        "company_id",
                        "member_id",
                        "debit_raw",
                        "debit_virtual",
                        "member_number",
                        "balance_raw",
                        "old_balance_raw",
                        "balance_virtual",
                        "old_balance_virtual",
                        "type_name"
                    ]
                ]
            );
    }

    public function testCustomerAccountTotal()
    {
        $this->post('/customer/account/total', ['client_number' => 6, 'member_id' => 1, 'point' => 100])
            ->seeStatusCode(200)
            ->seeJsonStructure(
                ['data' =>
                    ["current_balance"]
                ]
            );
    }

    public function testGetMemberPointTotalCredit()
    {
        $this->post('/customer/account/credit/total', ['client_number' => 6, 'member_id' => 1, 'point' => 100])
            ->seeStatusCode(200)
            ->seeJsonStructure(
                ['data' =>
                    ["credit"]
                ]
            );
    }

    public function testGetMemberPointTotalDebit()
    {
        $this->post('/customer/account/debit/total', ['client_number' => 6, 'member_id' => 1, 'point' => 100])
            ->seeStatusCode(200)
            ->seeJsonStructure(
                ['data' =>
                    ["debit"]
                ]
            );
    }

    public function testGetCustomersPlusAccount()
    {
        $this->get('/customer/list/account', ['client_number' => 6, 'member_id' => 1, 'point' => 100])
            ->seeStatusCode(200)
            ->seeJsonStructure(
                ['data' =>
                   ['*' => ["firstname",
                        "lastname",
                        "email",
                        "company_id",
                        "phone",
                        "address",
                        "account"]
                ]
                    ]
            );
    }
}